/* Deep Waters v0.9.0 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compose = require('./compose.js');
var every = require('./every.js');
var or = require('./or.js');
var when = require('./when.js');
var isPrimitive = require('./isPrimitive.js');
var isObject = require('./isObject.js');
var isDate = require('./isDate.js');

var objectEquals = function objectEquals(object) {
  return function (value) {
    if (!isObject.default(object)) return false;
    var objKeys = Object.keys(object);
    var valueKeys = Object.keys(value);
    if (objKeys.length !== valueKeys.length) return false; // eslint-disable-next-line no-use-before-define

    return valueKeys.every(function (key) {
      return equals(value[key])(object[key]);
    });
  };
}; // eslint-disable-next-line no-use-before-define


var arrayEquals = function arrayEquals(array) {
  return every.default(function (item, index) {
    return equals(item)(array[index]);
  });
};
/**
 * Takes a value val, of any type, and returns a new function evaluating that the received value equals val
 * (the one previously defined).
 * @param value
 * @returns {function(*=): boolean}
 */


var equals = function equals(value) {
  return compose.default(when.default(or.default(isPrimitive.default), function (string) {
    return value === string;
  }), when.default(isDate.default, function () {
    return !!value.valueOf();
  }), when.default(Array.isArray, arrayEquals(value)), when.default(isObject.default, objectEquals(value)));
};

exports.default = equals;
