/* Deep Waters v0.9.0 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/**
 * Takes an object of validators defining the possible shape of an object and returns a new function that reports
 * whether the received objects matches that shape or not.
 *
 * Please note:
 * The created validator isUser only makes sure the received object has all the defined keys and each key matches
 * the defined validation. It does not check on properties not defined by the shape, meaning it will return true
 * even if the object has additional properties.
 *
 * @param shape
 * @returns {function(*): boolean}
 */
var hasShape = function hasShape(shape) {
  return function (object) {
    return Object.keys(shape).every(function (prop) {
      var validator = shape[prop];
      var result = false;

      try {
        result = validator(object[prop]);
      } catch (e) {
        return false;
      }

      return result;
    });
  };
};

exports.default = hasShape;
